/*
 * (C) Copyright 2011
 * CompuLab Ltd. <www.compulab.co.il>
 * Ilya Ledvich <ilya@compulab.co.il>
 * Igor Grinberg <grinberg@compulab.co.il>
 *
 * X-Loader common board structures, functions and definitions
 * for the CM-T3x board family.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.
 */

#ifndef __CM_T3X_COMMON_H__
#define __CM_T3X_COMMON_H__

#include <part.h>

#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))

#define U_BOOT_FILENAME "u-boot.bin"

/* Output macros */
#ifdef CFG_PRINTF
# define pr_info	printf
# define pr_err		printf
#else
# define pr_info(fmt, ...)
# define pr_err(fmt, ...)
#endif

/* Debug output macros */
#if CM_T3X_DEBUG
# define pr_debug	pr_info
#else
# define pr_debug(fmt, ...)
#endif

/* CPU register read/write macros */
#define __raw_readl(a)		(*(volatile unsigned int *)(a))
#define __raw_writel(v, a)	(*(volatile unsigned int *)(a) = (v))
#define __raw_readw(a)		(*(volatile unsigned short *)(a))
#define __raw_writew(v, a)	(*(volatile unsigned short *)(a) = (v))

#ifdef DEBUG_LED
/* DEBUG LED GPIO pin offset in the GPIO bank */
#define DEBUG_LED_GPIO_OFFSET		(DEBUG_LED & 0x1F)
#define DEBUG_LED_GPIO_BANK		((DEBUG_LED >> 5) + 1)
#define DEBUG_LED_FCLKEN_REG	\
	(DEBUG_LED_GPIO_BANK == 1 ? CM_FCLKEN_WKUP : CM_FCLKEN_PER)
#define DEBUG_LED_ICLKEN_REG	\
	(DEBUG_LED_GPIO_BANK == 1 ? CM_ICLKEN_WKUP : CM_ICLKEN_PER)
#define DEBUG_LED_CLKEN_OFFSET	\
	(DEBUG_LED_GPIO_BANK == 1 ? 3 : DEBUG_LED_GPIO_BANK + 11)
void led_enable(void);
void led_blink(int times, int fast);
#else
static inline void led_enable(void) {}
static inline void led_blink(int times, int fast) {}
#endif

extern block_dev_desc_t *mmc_get_dev(int dev);

void try_unlock_memory(void);

u32 get_cpu_rev(void);
int cpu_has_iva(void);

/*********************************************************
 * Routine: delay
 * Description: spinning delay to use before udelay works
 *********************************************************/
static inline void delay(unsigned long loops)
{
	__asm__ volatile ("1:\n" "subs %0, %1, #1\n"
			  "bne 1b" : "=r" (loops) : "0"(loops));
}

#endif /* __CM_T3X_COMMON_H__ */
